#ifndef BOSS_H
#define BOSS_H

/**
 * \file Boss.h
 * \brief Déclaration de la class Boss
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */
 
#include "Ennemi.h"
#include "TypeBoss.h"

/**
 * \class Boss Boss.h
 * \brief Classe représentant les boss de fin de niveau
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */
 
class Boss : public Ennemi
{
	private:
		TypeBoss* my_type; /**< Type du bonus */

		sf::Clock my_changePatternClock; /**< Clock (sert à changer le pattern régulièrement) */
		float my_changePatternDelay; /**< Délai entre deux changements de pattern */

		ShootPattern my_actualShootPattern; /**< Pattern actuel */


	public:
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param type : Type du boss
		 */
		Boss(TypeBoss* type);
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_actualShootPattern
		 *
		 * \return : Le pattern actuellement utilisé
		 */
		ShootPattern getShootPattern() const;
		
		/**
		 * \brief Changer le pattern
		 *
		 * Méthode permettant de changer le pattern actuellement utilisé
		 */
		void changePattern();
		 
		/**
		 * \brief Déplacer le boss
		 *
		 * Méthode permettant de déplacer le Boss
		 */
		void move();

};

#endif
