#include "Ennemi.h"
#include "GameModel.h"
#include "constantes.h"
#include <cmath>
#include <iostream>

#define PI 3.14159265

using namespace std;
using namespace sf;

Ennemi::Ennemi(TypeEnnemi* type)
:Vaisseau(type->getHP(), type->getTypeTir(), type->getShootingSpeed())
{	my_type = type;
	my_dx = my_dy = type->getSpeed();
	my_w = type->getW() * ENEMY_SCALE;
	my_h = type->getH() * ENEMY_SCALE;
	my_x = rand() % (MODEL_WIDTH - my_w);
	my_y = -my_h;
		}

Ennemi::~Ennemi()
{}


int Ennemi::getScore() const
{return my_type->getScore();	}
TypeEnnemi* Ennemi::getType() const
{return my_type;	}
ShootPattern Ennemi::getShootPattern() const
{return my_type->getShootPattern();	}

void Ennemi::shoot(GameModel* model)
{
	if(my_clock.GetElapsedTime() > my_shootingSpeed )
	{
		switch(getShootPattern())
		{
			case SIMPLE :
			{
				Tir* t = new Tir(my_x, my_y, 0,  my_typeTir->getSpeed(), 0, my_typeTir);
				t->setX(my_x + my_w/2 - t->getW()/2);
				model->addTir(t);
			} break;

			case M_TRIPLE :
			{
				Tir* T1 = new Tir(my_x, my_y, -my_typeTir->getSpeed()/3, my_typeTir->getSpeed(),0, my_typeTir); // va en bas à gauche
				Tir* T2 = new Tir(my_x, my_y, 0, my_typeTir->getSpeed(),0, my_typeTir);			//tout droit
				Tir* T3 = new Tir(my_x, my_y, my_typeTir->getSpeed()/3, my_typeTir->getSpeed(),0, my_typeTir); //va en bas à droite
				T1->setX(my_x + my_w/2 - T1->getW()/2);
				T2->setX(my_x + my_w/2 - T2->getW()/2);
				T3->setX(my_x + my_w/2 - T3->getW()/2);
				model->addTir(T1);
				model->addTir(T2);
				model->addTir(T3);
			} break;

			case TOPLAYER : 
			{
				int DY = model->getPlayerShip()->getY() - my_y; //Distance entre l'ennemi et le joueur
				int DX = model->getPlayerShip()->getX() - my_x;

				float l = my_typeTir->getSpeed()/sqrt(pow(DX, 2) + pow(DY, 2)); //calcul du facteur entre la vitesse du tir et la distance entre l'ennemi et le joueur

				float dy = l * DY; //Calcul de x et y pour que la distance parcourue par le tir soit égal à celle prévue par le tir, même en diagonale.
				float dx = l * DX;
				
				Tir* t = new Tir(my_x, my_y, dx, abs(dy), 0, my_typeTir); //Prends la valeur absolue de dy, ce qui fait que le tir ne va vers le joueur que quand l'ennemi est devant lui
			    t->setX(my_x + my_w/2 - t->getW()/2);
				model->addTir(t);
			} break;

			case HALFCIRCLE : 
			{
				float rad;
				for(int  i = 0; i <= 180; i += 30)
				{
					rad = i * PI/180;
					Tir* t =  new Tir(my_x, my_y, cosf(rad) * my_typeTir->getSpeed(), sinf(rad) * my_typeTir->getSpeed(), 0, my_typeTir);
					t->setX(my_x + my_w/2 - t->getW()/2);
					model->addTir(t);
				}
			}break;
		}
		my_clock.Reset();
	}
}

vector<Tir*>::iterator Ennemi::isShot(GameModel* model) const
{
	vector<Tir*>::iterator shot = model->getVectorTir()->end();
    vector<Tir*>::iterator it = model->getVectorTir()->begin();
    while( it != model->getVectorTir()->end() && shot == model->getVectorTir()->end())
    {
        if((*it)->getX() + (*it)->getW() > my_x && (*it)->getX() < my_x + my_w && (*it)->getY() + (*it)->getH() > my_y && (*it)->getY() < my_y+my_h)
        {
	        if((*it)->getPlayer())
	           shot = it;  
        }  
        it++;
    }

   return shot;
}

void Ennemi::move()
{ moveDown();	}