#ifndef ENNEMI_H
#define ENNEMI_H

/**
 * \file Ennemi.h
 * \brief Déclaration de la class Ennemi
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

#include <SFML/Window.hpp>
#include "Vaisseau.h"
#include "TypeEnnemi.h"
#include "Horloge.h"

/**
 * \class Ennemi Ennemi.h
 * \brief Classe représentant les ennemis du joueur
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class Ennemi : public Vaisseau
{
	private :
		TypeEnnemi* my_type; /**< Type d'ennemi */

	public : 
	
		/**
		 * \brief Constructeur par défaut
		 */
		Ennemi();
		
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param type : Type d'ennemi
		 */
		Ennemi(TypeEnnemi* type);
		
		/**
		 * \brief Destructeur
		 */
		virtual ~Ennemi();
		
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_type.my_score
		 *
		 * \return : Les points rapportés par l'ennemi
		 */
		int getScore() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_type
		 *
		 * \return : Le type d'ennemi
		 */
		TypeEnnemi* getType() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_type.my_shootPattern
		 *
		 * \return : Le pattern utilisé par l'ennemi
		 */
		virtual ShootPattern getShootPattern() const;

		
		/**
		 * \brief Faire tirer l'ennemi
		 *
		 * \param model : Modéle du jeu
		 */
		void shoot(GameModel* model);
		
		/**
		 * \brief Indiquer le tir touchant l'ennemi
		 *
		 * \param model : Modéle du jeu
		 * \return : Le tir touchant l'ennemi
		 */
		std::vector<Tir*>::iterator isShot(GameModel* model) const;

		 
		/**
		 * \brief Déplacer l'ennemi
		 */
		void move();
};

#endif
