#include "GameModel.h"
#include "constantes.h"
#include <iostream>
#include <fstream>

GameModel::GameModel(int viesMax, int initialLives, int difficulty)
	:my_viesMax(viesMax), my_difficulty(difficulty)
{	
	my_score = 0;

	my_tirs_types[0] = new TypeTir(4,35,20,12, "images/bullet_player1.png"); 
	my_tirs_types[1] = new TypeTir(4,35,40,14, "images/bullet_player2.png");
	my_tirs_types[2] = new TypeTir(4,35,60,16, "images/bullet_player3.png");

	my_tirs_types[3] = new TypeTir(14,15,20,6, "images/bullet_ennemies.png");
	my_tirs_types[4] = new TypeTir(14,15,40,8, "images/bullet_ennemies2.png");
	my_tirs_types[5] = new TypeTir(14,15,60,10, "images/bullet_ennemies3.png");
	my_tirs_types[6] = new TypeTir(14,15,80,10, "images/bullet_ennemies4.png");

	my_ennemi_types[0] = new TypeEnnemi(100, 3, 0.5f, 20, 142,115,my_tirs_types[3], SIMPLE, "images/ennemy_0.png");
	my_ennemi_types[1] = new TypeEnnemi(200, 4, 0.7f, 40, 201,179,my_tirs_types[4], M_TRIPLE, "images/ennemy_1.png");
	my_ennemi_types[2] = new TypeEnnemi(300, 5, 0.4f, 60, 155,278,my_tirs_types[5], TOPLAYER, "images/ennemy_2.png");

	my_boss_type = new TypeBoss(500, 3, 0.5f, 500, 142 * 3, 115*3, my_tirs_types[6], HALFCIRCLE, "images/ennemy_0.png", 3);

	my_bonuses_types[0] = new TypeBonus(LIFE, 1, 0.1f, 1);
	my_bonuses_types[1] = new TypeBonus(POWER, 10, 0.5f, 1);
	my_bonuses_types[2] = new TypeBonus(BOMB, 1, 0.3f, 1);
	my_bonuses_types[3] = new TypeBonus(POWER, 100, 0.2f, 1.5f);
	my_bonuses_types[4] = new TypeBonus(HP, 80, 0.3f, 1);

	my_ship = new Joueur(0,my_tirs_types[0], initialLives, 500, 3, 0.2f);

	my_nextLevel = false;

	niveau.initNiveau(this);
	itNiveau = niveau.getMap()->begin();
}
	
GameModel::~GameModel()
{	
	for(int i = 0; i < NB_TIRS_TYPES; i++)
		delete my_tirs_types[i];
	for(int i = 0; i < NB_ENNEMY_TYPES; i++)
		delete my_ennemi_types[i];	
	for(int i = 0; i < NB_BONUS_TYPES; i++)
		delete my_bonuses_types[i];

	for(vector<Tir*>::iterator it = my_tirs.begin(); it != my_tirs.end(); it++)
		delete *it;
	for(vector<Ennemi*>::iterator it = my_ennemis.begin(); it != my_ennemis.end(); it++)
		delete *it;
	for(vector<Bonus*>::iterator it = my_bonuses.begin(); it != my_bonuses.end(); it++)
		delete *it;

	delete my_ship;
	delete my_boss_type;
}

void GameModel::nextStep()
{

	//gestion du niveau et de l'apparition des ennemis
	if(my_clock.getTime() > itNiveau->first)
	{
		if(itNiveau != niveau.getMap()->end())
		{	addEnnemy(itNiveau->second);
			itNiveau++;	}
		else
			if(my_ennemis.empty())
				changeNiveau();
	}

	vector<Tir*>::iterator i; //itérateur sur vector<Tir> qui va servir à plusieurs choses

	for(i = my_tirs.begin(); i != my_tirs.end(); i++) //On s'en sert d'abord pour déplacer tout les tirs
	{
			(*i)->move();
			if((*i)->getY() < 0 || (*i)->getY() > MODEL_HEIGHT || (*i)->getX() < 0 || (*i)->getX() > MODEL_WIDTH)
			{	my_deletedItems.push_back(*i);
				delete *i;
				i = my_tirs.erase(i);
				i--;	}	
	}

	i = my_ship->isShot(this); // on se ressert de l'itérateur pour vérifier si on est touché
	if(i != my_tirs.end())
	{	
		my_ship->setHP(my_ship->getHP() - (*i)->getType()->getDamages());
		my_deletedItems.push_back(*i);
		delete *i;
		my_tirs.erase(i);
	}
	if(my_ship->isHitByEnemy(this) || my_ship->getHP() <= 0)
		gameOver();

	//Puis on s'occupe des ennemis
	for(vector<Ennemi*>::iterator itE = my_ennemis.begin(); itE != my_ennemis.end(); itE++)
	{ 
		if((*itE)->getAlive())
		{
			Boss* b = dynamic_cast<Boss*>(*itE);
			if(b != NULL)
				b->changePattern();

			(*itE)->move(); //On déplace l'énnemi
			(*itE)->shoot(this); //On fait tirer l'ennemi

			// On se ressert de l'itérateur pour cette fois vérifier si les ennemis sont touchés
			i = (*itE)->isShot(this); 
			if( i != my_tirs.end())
			{
				(*itE)->setHP((*itE)->getHP() - (*i)->getType()->getDamages());
				my_deletedItems.push_back(*i);
				delete *i;
				my_tirs.erase(i);

				if((*itE)->getHP() <=0)
				{
					my_score += (*itE)->getScore();
					addBonus((*itE)->getX(), (*itE)->getY());
					(*itE)->setAlive(false);
					itE--;
				}
			}
			else if((*itE)->getY() > MODEL_HEIGHT)
			{	my_deletedItems.push_back(*itE);
				delete *itE;
				my_ennemis.erase(itE);
				itE--;
			}
		}
	}	

	//Et enfin des bonuses
	for(vector<Bonus*>::iterator itB = my_bonuses.begin(); itB != my_bonuses.end(); itB++)
	{
		(*itB)->moveDown();
		if((*itB)->getX() > MODEL_HEIGHT)
		{
			my_deletedItems.push_back(*itB);
			my_bonuses.erase(itB);
			itB--;
		}
		else if((*itB)->getX() + (*itB)->getW() > my_ship->getX() && (*itB)->getX() < my_ship->getX()+ my_ship->getW() 
			&& (*itB)->getY() + (*itB)->getH() > my_ship->getY() && (*itB)->getY() < my_ship->getY() + my_ship->getH())
		{
			switch((*itB)->getType()->getEffect())
			{
				case LIFE :
					if(my_ship->getVies() + 1 <= my_viesMax) 
						my_ship->setVies(my_ship->getVies() + (*itB)->getType()->getValue()); 
					break;
				case BOMB : my_ship->setBombs(my_ship->getBombs() + (*itB)->getType()->getValue()); break;
				case POWER : my_ship->addPower((*itB)->getType()->getValue(), this); break;
				case HP : my_ship->addHP((*itB)->getType()->getValue() ); break;
			}

			my_deletedItems.push_back(*itB);
			my_bonuses.erase(itB);
			itB--;
		}
	}
}

void GameModel::addEnnemy(TypeEnnemi* type)
{
	TypeBoss* tb = dynamic_cast<TypeBoss*>(type);
	if(tb != NULL)
	{	
		Boss *b = new Boss(tb);
		my_ennemis.push_back(b);
	}
	else
	{	
		Ennemi *e = new Ennemi(type);
		my_ennemis.push_back(e);
	}
}

void GameModel::addTir(Tir* t)
{	my_tirs.push_back(t);	}

void GameModel::addBonus(int x, int y)
{
	//Choisit un bonus au hasard et puis fait un test pour appliquer sa proba
	int bonusType = rand() % NB_BONUS_TYPES;
	if(rand()/(float)RAND_MAX <= my_bonuses_types[bonusType]->getFrequency())
	{
		Bonus* b = new Bonus(x, y, my_bonuses_types[bonusType]);
		my_bonuses.push_back(b);
	}
}

void GameModel::removeEnnemy(Ennemi* ennemi)
{
	vector<Ennemi*>::iterator it = my_ennemis.begin();
	bool found = false;
	while(it != my_ennemis.end() && !found)
	{
		if(*it == ennemi)
			found = true;
		it++;
	}
	if(found)
		my_ennemis.erase(it);
}
void GameModel::removeEnnemy(vector<Ennemi*>::iterator it)
{my_ennemis.erase(it);	}

void GameModel::saveScore(string name)
{
	multimap<int, string> scores; //Utilisation d'une multimap car elle se trie toute seule et peut contenir plusieurs éléments qui ont la même clé (ici, plusieurs personnes qui ont le même score)
	ifstream f("highScores.txt"); // Ouverture en lecture

	if(f)
	{
		int n;
		string s;

		while(f>> s && f>>n)
			scores.insert(pair<int, string>(n, s)); //On charge les scores du fichiers dans la map.

		f.close();
	}
	else
		cout << "Fichier highScores.txt inexistant, création d'un neuf" << endl;

	scores.insert(pair<int, string>(my_score, name)); // On ajoute le score à ajouter, qui va s'insérer à la bonne position

	ofstream f2("highScores.txt"); //Ouverture en écriture

	int cpt = 0;
	if(f2)
		for(multimap<int,string>::reverse_iterator it =  scores.rbegin(); it != scores.rend() && cpt < 20; it++,  cpt++) //On n'enregistre que 20 scores aux maximum
			f2 << it->second <<" " << it->first << endl; //on remet les score triés dans le fichier.
	else
		cout << "error" << endl;
}

void GameModel::changeNiveau()
{
	niveau.initNiveau(this);
	itNiveau = niveau.getMap()->begin();
	for(vector<Tir*>::iterator it = my_tirs.begin(); it != my_tirs.end(); it++)
	{	my_deletedItems.push_back(*it);
		delete *it;	}
	my_tirs.clear();
	my_nextLevel = true;
	my_clock.reset();

	//Redéfinition du type de boss pour qu'il s'adapte au niveau en cours
	*my_boss_type = TypeBoss(500 * getLevelID(), 2 * getLevelID(), 0.5f, 500 + (200* getLevelID()), 142 * 3, 115*3, my_tirs_types[6], HALFCIRCLE, "images/ennemy_0.png", 3);			
}

void GameModel::gameOver()
{
	my_ship->setVies(my_ship->getVies() - 1);
	my_ship->setHP(my_ship->getMaxHP());
	my_ship->setX(MODEL_WIDTH/2 - my_ship->getW()/2);
	my_ship->setY(MODEL_HEIGHT - my_ship->getH());
	for(vector<Tir*>::iterator it = my_tirs.begin(); it != my_tirs.end(); it++)
	{	my_deletedItems.push_back(*it);
		delete *it;	}
	my_tirs.clear();
	for(vector<Bonus*>::iterator it = my_bonuses.begin(); it != my_bonuses.end(); it++)
	{	my_deletedItems.push_back(*it);
		delete *it;	}
	my_bonuses.clear();
	my_ship->setAlive(false);
}


//Accesseurs
		
void GameModel::getShipPos(int &x, int &y) const 
{
  x = my_ship ->getX();
  y = my_ship ->getY();
}

bool GameModel::getNextLevel() const
{return my_nextLevel;	}
int GameModel::getLevelID() const
{return niveau.getID();	}
int GameModel::getScore() const
{return my_score;	}
int GameModel::getDifficulty() const
{return my_difficulty;	}

string GameModel::getStringLevelID() const
{
	stringstream str;
	str << niveau.getID() << endl;
	return str.str();
}

string GameModel::getStringScore() const
{
	stringstream str;
	str << my_score << endl;
	return str.str();
}

TypeEnnemi* const* GameModel::getEnnemyTypes() const
{return my_ennemi_types;	}
TypeTir* const*  GameModel::getTirTypes() const
{return my_tirs_types;	}
TypeBoss* GameModel::getBossType() const
{return my_boss_type;	}

Joueur*  GameModel::getPlayerShip() const
{	return my_ship;}


vector<Ennemi*>* GameModel::getVectorEnnemis()
{return &my_ennemis;	}
vector<Tir*>* GameModel::getVectorTir() 
{return &my_tirs;	}
const vector<Bonus*> & GameModel::getVectorBonuses()
{return my_bonuses;	}
vector<MovableElement*>* GameModel::getDeletedItems()
{return &my_deletedItems;	}

Horloge*  GameModel::getClock()
{ return &my_clock;	}


void GameModel::setScore(int score)
{my_score = score;	}
void GameModel::setNextLevel(bool nextLvl)
{my_nextLevel = nextLvl;	}
