#include "GameView.h"
#include "GameModel.h"
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;
using namespace sf;

GameView::GameView(int w, int h, int bpp): my_w(w), my_h(h){
	
  my_model = NULL;

  my_window = new RenderWindow(sf::VideoMode(w, h, bpp), "Space Sheep XTrem Turbo", sf::Style::Close);
  my_window->UseVerticalSync(true);

  my_langue = 1;
  updateLanguage();

  my_background_image = new sf::Image();
  my_background_parallax = new sf::Image();
  my_ship_image = new sf::Image();
  my_shoot_image = new sf::Image();
  my_boom_image = new sf::Image();
  my_bonus_image = new sf::Image();

  my_boom_image->LoadFromFile("images/explosion.png");

  //Initialisation des sprites
  if (!my_background_image->LoadFromFile("images/space.png") ||
      !my_background_parallax->LoadFromFile("images/stars.png") ||
      !my_ship_image->LoadFromFile("images/ship_all.png")	||
      !my_bonus_image->LoadFromFile("images/bonus.png"))
  {
    my_background_sprite = GraphicElement();
    my_background_parallax_sprite = GraphicElement();
  }
  else {
    my_background_image->SetSmooth(false);
    my_background_sprite = GraphicElement(my_background_image, 0,0,w,h);
    my_background_sprite2 = GraphicElement(my_background_image,0,-h,w,h);

    my_background_parallax->SetSmooth(false);
    my_background_parallax_sprite = GraphicElement(my_background_parallax, 0, 0, w, h);
    my_background_parallax_sprite2 = GraphicElement(my_background_parallax, 0, -h, w, h);
  }

  //Initialisation des sons

  my_soundLevel = 100;
  my_soundpack = 1;

  my_titlethemeBuffer = new sf::SoundBuffer();
  my_deathmonsterBuffer = new sf::SoundBuffer();
  my_deathbossBuffer = new sf::SoundBuffer();
  my_nextLevelBuffer = new sf::SoundBuffer();
  my_gameoverBuffer = new sf::SoundBuffer();
  my_shotBuffer = new sf::SoundBuffer();

 updateSound();

}

GameView::~GameView(){
  if(my_window!= NULL)
    delete my_window;
   
  delete my_background_image;
  delete my_background_parallax;
  delete my_ship_image;
  delete my_shoot_image;
  delete my_boom_image;
  delete my_bonus_image;

  delete my_titlethemeBuffer;
  delete my_deathmonsterBuffer;
  delete my_nextLevelBuffer;
  delete my_gameoverBuffer;
  delete my_shotBuffer;

  for(map<TypeEnnemi* const, sf::Image*>::iterator it = my_ennemies_images.begin(); it!= my_ennemies_images.end(); it++)
    delete it->second;

  for(map<TypeTir* const, sf::Image*>::iterator it = my_tirs_images.begin(); it != my_tirs_images.end(); it++)
    delete it->second;

  for(map<MovableElement * const, GraphicElement *>::iterator it = my_elementToGraphicElement.begin(); it != my_elementToGraphicElement.end(); it++)
    delete it->second;
}

void GameView::setModel(GameModel * model){

  // On clear les maps synchronisée avec l'ancien modèle, le nouveau n'ayant pas forcément les mêmes adresses
  if(my_model != NULL)  
  {
    for(map<TypeEnnemi* const, sf::Image*>::iterator it = my_ennemies_images.begin(); it!= my_ennemies_images.end(); it++)
      delete it->second;

    my_ennemies_images.clear();

    for(map<TypeTir* const, sf::Image*>::iterator it = my_tirs_images.begin(); it != my_tirs_images.end(); it++)
      delete it->second;

    my_tirs_images.clear();

    for(map<MovableElement* const, GraphicElement*>::iterator it = my_elementToGraphicElement.begin(); it != my_elementToGraphicElement.end(); it++)
      delete it->second;

    my_elementToGraphicElement.clear();
  }

  my_model = model;

  my_elementToGraphicElement[my_model->getPlayerShip()] = new GraphicElement (my_ship_image,0,0, my_ship_image->GetWidth(),my_ship_image->GetHeight());
  my_elementToGraphicElement[my_model->getPlayerShip()]->SetScale(SHEEP_SCALE, SHEEP_SCALE);
  my_elementToGraphicElement[my_model->getPlayerShip()]->SetSubRect(sf::IntRect(0,0,SHEEP_WIDTH, SHEEP_HEIGHT)); 

  for(int i = 0; i < NB_ENNEMY_TYPES; i++)
  {
     my_ennemies_images[my_model->getEnnemyTypes()[i]] = new sf::Image();
     my_ennemies_images[my_model->getEnnemyTypes()[i]]->LoadFromFile(my_model->getEnnemyTypes()[i]->getFilename());
  }
  my_ennemies_images[my_model->getBossType()] = new sf::Image();
  my_ennemies_images[my_model->getBossType()]->LoadFromFile(my_model->getBossType()->getFilename());

  for(int i = 0; i < NB_TIRS_TYPES; i++)
  {
    my_tirs_images[my_model->getTirTypes()[i]] = new sf::Image();
    my_tirs_images[my_model->getTirTypes()[i]]->LoadFromFile(my_model->getTirTypes()[i]->getFilename());
  }
}

void GameView::updateLanguage()
{
  string langue;
  switch(my_langue)
  {
    case 1 : langue = "French"; break;
    case 2 : langue = "English"; break;
  }

  string filename = "LanguageFiles/" + langue + ".txt";
  ifstream f(filename.c_str());

  if(f)
  {
    string s;

    for(int i = 0; i < NB_STRINGS; i++)
      getline(f,my_strings[i]); 

  }
}

void GameView::updateSound()
{
  string soundpack, extension;
  switch(my_soundpack)
  {
    case 1 : soundpack = "8bit"; extension = ".wav" ;break;
    case 2 : soundpack = "Human"; extension = ".ogg" ;break;
  }

  string directoryPath = "Music/" + soundpack + "/" ;

  *my_titlethemeBuffer =  sf::SoundBuffer();
  *my_deathmonsterBuffer =  sf::SoundBuffer();
  *my_deathbossBuffer =  sf::SoundBuffer();
  *my_nextLevelBuffer =  sf::SoundBuffer();
  *my_gameoverBuffer =  sf::SoundBuffer();
  *my_shotBuffer =  sf::SoundBuffer();

  if(!my_titlethemeBuffer->LoadFromFile(directoryPath + "title" + extension) ||
  !my_maintheme.OpenFromFile(directoryPath + "maintheme" + extension) ||
  !my_deathmonsterBuffer->LoadFromFile(directoryPath + "deathmonster" + extension) ||
  !my_deathbossBuffer->LoadFromFile(directoryPath + "deathboss" + extension) ||
  !my_nextLevelBuffer->LoadFromFile(directoryPath + "nextlevel" + extension) ||
  !my_gameoverBuffer->LoadFromFile(directoryPath + "gameover" + extension) ||
  !my_shotBuffer->LoadFromFile(directoryPath + "shot" + extension) )
  {
    my_titletheme = Sound();
    my_deathmonster = Sound();
    my_deathboss = Sound();
    my_nextLevel = Sound();
    my_gameover = Sound();
    my_shotsound =  Sound();
  }
  else
  {  
    my_maintheme.SetLoop(true);
    my_titletheme = Sound(*my_titlethemeBuffer);
    my_nextLevel = Sound(*my_nextLevelBuffer);
    my_deathmonster = Sound(*my_deathmonsterBuffer);
    my_deathboss = Sound(*my_deathbossBuffer);
    my_gameover = Sound(*my_gameoverBuffer);
    my_shotsound = Sound(*my_shotBuffer);
  }

}

void GameView::updateVolume()
{
  my_titletheme.SetVolume(my_soundLevel);
  my_maintheme.SetVolume(my_soundLevel);
  my_nextLevel.SetVolume(my_soundLevel);
  my_deathmonster.SetVolume(my_soundLevel);
  my_gameover.SetVolume(my_soundLevel);
  my_shotsound.SetVolume(my_soundLevel);
}

void GameView::bgscroll()
{
  my_background_sprite.Move(0,1);
  my_background_sprite2.Move(0,1);
  if(my_background_sprite.GetPosition().y == VIEW_HEIGHT)
    my_background_sprite.SetPosition(0,-my_h);
  if(my_background_sprite2.GetPosition().y == VIEW_HEIGHT)
    my_background_sprite2.SetPosition(0,-my_h);

  my_background_parallax_sprite.Move(0, 2);
  my_background_parallax_sprite2.Move(0, 2);
  if(my_background_parallax_sprite.GetPosition().y == VIEW_HEIGHT)
      my_background_parallax_sprite.SetPosition(0, -my_h);
  if(my_background_parallax_sprite2.GetPosition().y == VIEW_HEIGHT)
      my_background_parallax_sprite2.SetPosition(0, -my_h);
}

void GameView::explosion(GraphicElement &elem, Ennemi* adr)
{
  if(elem.GetImage() != my_boom_image)
  { 
    elem = GraphicElement(my_boom_image, elem.GetPosition().x,elem.GetPosition().y, my_boom_image->GetWidth(), my_boom_image->GetHeight());
    elem.SetSubRect(sf::IntRect(0,0, 200,205));

    Boss* b = dynamic_cast<Boss*>(adr);
    if(b != NULL)
      my_deathboss.Play();
    else
      my_deathmonster.Play();
  }
  else
    elem.SetSubRect(sf::IntRect(elem.GetSubRect().Left + 200, 0, elem.GetSubRect().Right + 200, 205));

}

void GameView::draw()
{
  if(my_titletheme.GetStatus() != sf::Sound::Stopped)
    my_titletheme.Stop();
  if(my_maintheme.GetStatus() != sf::Sound::Playing)
    my_maintheme.Play();

  my_window->Clear();

  if(my_model->getPlayerShip()->getAlive()) //Si vivant
  {
    if(!my_model->getNextLevel())
    {
      bgscroll();
      my_window->Draw(my_background_sprite);
      my_window->Draw(my_background_sprite2);
      my_window->Draw(my_background_parallax_sprite);
      my_window->Draw(my_background_parallax_sprite2);

      for(map<MovableElement * const, GraphicElement *>::iterator it = my_elementToGraphicElement.begin(); it!=my_elementToGraphicElement.end();it++)
        if(it->second->getVisible())
          it->second->draw(my_window);

      my_elementToGraphicElement[my_model->getPlayerShip()]->draw(my_window);
      my_hud.drawHUD(my_window);
      
    }
    else
    {
      my_maintheme.Pause();
      my_nextLevel.Play();
      transition(my_strings[FINISHLEVEL], 3);

      my_model->setNextLevel(false);
    }
  }
  else
  {
    if(my_model->getPlayerShip()->getVies() == 0 ) //Si plus de vies->transition game over
    {
      my_maintheme.Pause();
      my_gameover.Play();
      transition(my_strings[GAMEOVER], 3 );

      askName();
    }
    else // Si il reste encore des vies -> écran de transition de perte de vie
    {
      transition(my_strings[DEAD], 3 );

      my_model->getPlayerShip()->setAlive(true);
    }
  }
}

void GameView::Display()
{
  my_window->Display();
}

void GameView::transition(string text, float t)
{
  my_model->getClock()->pause();

  sf::String sText = String(text);
  sText.SetFont(sf::Font::GetDefaultFont());
  sText.SetPosition(VIEW_WIDTH/2 - sText.GetRect().GetWidth()/2,VIEW_HEIGHT/2 - sText.GetRect().GetHeight()/2);
  sText.SetColor(Color(255,255,255));

  sf::String sCount = String();

  sf::Clock countdown;
  
  while(countdown.GetElapsedTime() < t)
  {

    int IntCountDown = countdown.GetElapsedTime() ;
    my_window->Clear();

    stringstream sstr;
    sstr << t - IntCountDown << endl;

    sCount.SetText(sstr.str()); 
    sCount.SetPosition(VIEW_WIDTH/2 - sCount.GetRect().GetWidth()/2, VIEW_HEIGHT - 150);

    my_window->Draw(sText);
    my_window->Draw(sCount);
    my_window->Display();
  }

  my_model->getClock()->start();
}

void GameView::synchronize()
{
  //synchro du joueur
	int x_ship, y_ship;
  my_model->getShipPos(x_ship, y_ship);
  my_elementToGraphicElement[my_model->getPlayerShip()]->SetPosition(x_ship, y_ship);

  //update du HUD
  my_hud.update(my_model, my_strings);

  //synchro des tirs
  for(vector<Tir*>::iterator it = my_model->getVectorTir()->begin(); it != my_model->getVectorTir()->end(); it++)
  {
    if(my_elementToGraphicElement.count(*it) == 0)
      my_elementToGraphicElement[*it] = new GraphicElement();

    my_elementToGraphicElement[*it]->setVisible(true);

    /*Vu qu'on détruit les élements de la map que dans le destrcteur, si dans le modèle quand on crée un élement,
     il prend une adresse déjà référencée, la map l'associe au GraphicElement déjà existant(qui peut être n'importe quoi),
     du coup faut redéfinir le GraphicElement */
    *(my_elementToGraphicElement[*it]) = GraphicElement(my_tirs_images[(*it)->getType()], 0,0,my_tirs_images[(*it)->getType()]->GetWidth(), my_tirs_images[(*it)->getType()]->GetHeight());
    my_elementToGraphicElement[*it]->resize((*it)->getW(), (*it)->getH());
    my_elementToGraphicElement[*it]->SetPosition((*it)->getX(), (*it)->getY());

    if(!(*it)->getPlayer())
    {
      my_elementToGraphicElement[*it]->SetRotation(180);
      my_elementToGraphicElement[*it]->Move(my_elementToGraphicElement[*it]->GetSize().x, my_elementToGraphicElement[*it]->GetSize().y);
    }
    else
      my_elementToGraphicElement[*it]->SetRotation(0);
  }


  //synchro des ennemis
  for(vector<Ennemi*>::iterator itE = my_model->getVectorEnnemis()->begin(); itE!= my_model->getVectorEnnemis()->end(); itE++)
  { 
    if(my_elementToGraphicElement.count(*itE) == 0)
      my_elementToGraphicElement[*itE] = new GraphicElement();
    my_elementToGraphicElement[*itE]->setVisible(true);

    Boss* b = dynamic_cast<Boss*>(*itE);
    if(b != NULL)
      my_hud.updateBossPV(b);

    if((*itE)->getAlive())
    {
      //Idem qu'au dessus
      *(my_elementToGraphicElement[*itE]) = GraphicElement(my_ennemies_images[(*itE)->getType()], 0, 0 ,my_ennemies_images[(*itE)->getType()]->GetWidth(), my_ennemies_images[(*itE)->getType()]->GetHeight());
      
      my_elementToGraphicElement[*itE]->resize((*itE)->getW(), (*itE)->getH());
      my_elementToGraphicElement[*itE]->SetPosition((*itE)->getX(), (*itE)->getY());
    }
    else
    {
      if(my_elementToGraphicElement[*itE]->GetSubRect().Right == 600) // Si l'explosion est finie, on supprime
      { my_model->getDeletedItems()->push_back(*itE);
        delete *itE;
        my_model->removeEnnemy(itE);
        itE--;  }
      else // Sinon, on la continue
      {
        if((*itE)->getClock()->GetElapsedTime() > 0.2)
        {
          explosion(*(my_elementToGraphicElement[*itE]), *itE);
          my_elementToGraphicElement[*itE]->SetScale(0.5, 0.5);
          my_elementToGraphicElement[*itE]->SetPosition((*itE)->getX() - 50 + (*itE)->getW()/2, (*itE)->getY() - 51 + (*itE)->getH()/2);
          (*itE)->getClock()->Reset();
        }
      }
    }
  }


  //synchro des Bonus
  for(vector<Bonus*>::const_iterator itB = my_model->getVectorBonuses().begin(); itB != my_model->getVectorBonuses().end(); itB++)
  {
    if(my_elementToGraphicElement.count(*itB) == 0)
      my_elementToGraphicElement[*itB] = new GraphicElement();

    my_elementToGraphicElement[*itB]->setVisible(true);

    *(my_elementToGraphicElement[*itB]) = GraphicElement(my_bonus_image, 0, 0, my_bonus_image->GetWidth(), my_bonus_image->GetHeight());
    my_elementToGraphicElement[*itB]->SetPosition((*itB)->getX(), (*itB)->getY());
    my_elementToGraphicElement[*itB]->resize((*itB)->getW(), (*itB)->getH());
    switch((*itB)->getType()->getEffect())
    {
      case LIFE : my_elementToGraphicElement[*itB]->SetColor(sf::Color::Green); break;
      case POWER: my_elementToGraphicElement[*itB]->SetColor(sf::Color::Red); break;
      case BOMB : my_elementToGraphicElement[*itB]->SetColor(sf::Color::Blue); break;
      case HP : my_elementToGraphicElement[*itB]->SetColor(Color(255,20,147)); break;
    }
  }

  //On rend invisible les GraphicElements de la map dont la key n'est ni l'adresse d'un ennemi, ni celle d'un tir, ni celle du joueur

  for(vector<MovableElement*>::iterator it = my_model->getDeletedItems()->begin(); it != my_model->getDeletedItems()->end(); it++)
  {  
    /*Met seulement invisible, pour éviter d'avoir à faire des new et delete, et permettre de récupérer le GraphicElement si l'adresse est réallouée,
      mais provoque des erreurs de segmentations par moment, pour une raison inconnue */
    // my_elementToGraphicElement[*it]->setVisible(false); 


    /* Moins efficace que la méthode du dessus car fait un delete sur chaque élément détruit, et donc provoquera un new si l'adresse est réallouée,
      mais ne provoque pas d'erreur de segmentation */
     delete my_elementToGraphicElement[*it];
     my_elementToGraphicElement.erase(*it);
  }

  my_model->getDeletedItems()->clear();
}


bool GameView::treatEvents(bool &quit){
  bool result = false;
  const sf::Input& Input = my_window->GetInput();
  
  if(my_window->IsOpened())
  {
    result = true;
    Event event;
    while (my_window->GetEvent(event)) 
    {
  		if (event.Type == Event::Closed)
      {
  			my_window->Close();
  			result = false;
        quit = true;
			}
      if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Escape))
      {
        my_model->getClock()->pause();
        switch(pause())
        {
          case 1 : my_model->getClock()->start(); break;
          case 2 : my_maintheme.Pause();
                    result = false; 
                    break;
          case 3 : my_window->Close();
                    result = false;
                    quit = true;
                    break;
        }
      }
    }

    my_elementToGraphicElement[my_model->getPlayerShip()]->SetSubRect(sf::IntRect(0,0,SHEEP_WIDTH, SHEEP_HEIGHT));
      
    if(Input.IsKeyDown(sf::Key::Left))
    { my_model->getPlayerShip()->moveLeft();
      my_elementToGraphicElement[my_model->getPlayerShip()]->SetSubRect(sf::IntRect(200,0,400, SHEEP_HEIGHT)); }
  	if(Input.IsKeyDown(sf::Key::Right))
  	{	my_model->getPlayerShip()->moveRight();
      my_elementToGraphicElement[my_model->getPlayerShip()]->SetSubRect(sf::IntRect(400,0,600, SHEEP_HEIGHT)); }
  	if(Input.IsKeyDown(sf::Key::Up))
  		my_model->getPlayerShip()->moveUp();
  	if(Input.IsKeyDown(sf::Key::Down))
  		my_model->getPlayerShip()->moveDown();
  	
  	if(my_clock.GetElapsedTime() > 0.2)
  	{	
  		if(Input.IsKeyDown(sf::Key::Space))
  		{	my_model->getPlayerShip()->shoot(my_model);
        my_shotsound.Play();
        my_clock.Reset();}
      if(Input.IsKeyDown(sf::Key::B))
        {my_model->getPlayerShip()->useBomb(my_model);
          my_clock.Reset(); }

      if(Input.IsKeyDown(sf::Key::E))
      {  my_model->addEnnemy(my_model->getEnnemyTypes()[1]);
        my_clock.Reset(); }
      if(Input.IsKeyDown(sf::Key::Z))
       { my_model->addEnnemy(my_model->getEnnemyTypes()[0]);
        my_clock.Reset(); }
      if(Input.IsKeyDown(sf::Key::R))
       { my_model->addEnnemy(my_model->getEnnemyTypes()[2]);
        my_clock.Reset(); }
  	}


    if(my_model->getPlayerShip()->getVies() == 0 )
    {
      result = false;
    }
  }  
  return result;
}





int GameView::pause()
{
  sf::String sPause = String(my_strings[PAUSE]);
  sPause.SetPosition(VIEW_WIDTH/2 - sPause.GetRect().GetWidth()/2, 150);

  sf::String sResume = String(my_strings[RESUME]);
  sResume.SetPosition(VIEW_WIDTH/2 - sResume.GetRect().GetWidth()/2, 250);

  sf::String sBack = String(my_strings[BACKTOMENU]);
  sBack.SetPosition(VIEW_WIDTH/2 - sBack.GetRect().GetWidth()/2, 300);

  sf::String sQuit = String(my_strings[QUIT]);
  sQuit.SetPosition(VIEW_WIDTH/2 - sQuit.GetRect().GetWidth()/2, 350);

  sf::Shape rectPause = sf::Shape::Rectangle(100, 100, VIEW_WIDTH - 100, VIEW_HEIGHT - 100, sf::Color::Black, 3, sf::Color::White);

  int result = 0;
  int choice = 1;

  if(my_window->IsOpened())
  {
    Event event;

    while(result == 0)
    {
      my_window->Clear();
      draw();

      sResume.SetColor(sf::Color::White);
      sBack.SetColor(sf::Color::White);
      sQuit.SetColor(sf::Color::White);

      switch(choice)
      {
        case 1 : sResume.SetColor(sf::Color::Red); break;
        case 2 : sBack.SetColor(sf::Color::Red); break;
        case 3 : sQuit.SetColor(sf::Color::Red); break;
      }

      my_window->Draw(rectPause);
      my_window->Draw(sPause);
      my_window->Draw(sResume);
      my_window->Draw(sBack);
      my_window->Draw(sQuit);
      my_window->Display();

      while(my_window->GetEvent(event))
      {
        if (event.Type == Event::Closed)
        {
          result = 3;
        } 
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Up))
        {
          if(choice - 1 > 0)
            choice--;
        }
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Down))
        {
          if(choice + 1 <= 3)
            choice++;
        }
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Return))
          result = choice;
      }
    }
  }
  return result;
}


void GameView::intro()
{
  sf::String names = String(L"Harscoët-Chassaigne Productions");
  names.SetFont(sf::Font::GetDefaultFont());
  names.SetPosition(VIEW_WIDTH/2 - names.GetRect().GetWidth()/2, VIEW_HEIGHT/2 - names.GetRect().GetHeight());

  sf::String presents = String("Presents");
  presents.SetFont(sf::Font::GetDefaultFont());
  presents.SetPosition(VIEW_WIDTH/2 - presents.GetRect().GetWidth()/2, VIEW_HEIGHT/2);

  int alpha = 0;
  for(alpha = 0; alpha <= 255; alpha+= 2)
  {
    my_window->Clear();
    names.SetColor(Color(255,255,255,alpha));
    presents.SetColor(Color(255,255,255,alpha));
    my_window->Draw(names);
    my_window->Draw(presents);
    my_window->Display();
  }
  for(alpha = 255; alpha >= 0; alpha-=2)
  {
    my_window->Clear();
    names.SetColor(Color(255,255,255,alpha));
    presents.SetColor(Color(255,255,255,alpha));
    my_window->Draw(names);
    my_window->Draw(presents);
    my_window->Display();
  }
}

int GameView::menu()
{

  sf::Font shmupFont;
  shmupFont.LoadFromFile("Fonts/Spaceman.ttf");

  sf::String shmup = String("Space Sheep Xtrem Turbo \n       Penguin Edition");
  shmup.SetFont(shmupFont);
  shmup.SetSize(25);
  shmup.SetPosition(VIEW_WIDTH/2 - shmup.GetRect().GetWidth()/2, 50);
  shmup.SetColor(Color(0,0,0));

  sf::String play = String(my_strings[JOUER]);
  play.SetFont(sf::Font::GetDefaultFont());
  play.SetPosition(50,200);

  sf::String highscores = String(my_strings[HIGHSCORES]);
  highscores.SetPosition(50, 300);

  sf::String settings = String(my_strings[SETTINGS]);
  settings.SetPosition(50, 400);

  sf::String quit = String(my_strings[QUIT]);
  quit.SetPosition(50,500);

  if(my_titletheme.GetStatus() != sf::Sound::Playing)
    my_titletheme.Play();



  int result = 0;
  int choice = 1;

  if(my_window->IsOpened())
  {
    Event event;

    while(result == 0)
    {
      my_window->Clear(Color(255,255,255));
      my_window->Draw(shmup);

      play.SetColor(sf::Color::Black);
      highscores.SetColor(sf::Color::Black);
      settings.SetColor(sf::Color::Black);
      quit.SetColor(sf::Color::Black);

      switch(choice)
      {
        case 1 : play.SetColor(sf::Color::Red); break;
        case 2 : highscores.SetColor(sf::Color::Red); break;
        case 3 : settings.SetColor(sf::Color::Red); break;
        case 4 : quit.SetColor(sf::Color::Red); break;
      }

      my_window->Draw(play);
      my_window->Draw(highscores);
      my_window->Draw(settings);
      my_window->Draw(quit);
      my_window->Display();

      while(my_window->GetEvent(event))
      {
        if ((event.Type == Event::Closed) ||
        ((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Escape)))
        {
          my_window->Close();
          result = 4;
        } 
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Up))
        {
          if(choice - 1 > 0)
            choice--;
        }
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Down))
        {
          if(choice + 1 <= 4)
            choice++;
        }
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Return))
          result = choice;
      }
    }
  }
  return result;
}

void GameView::options(bool &quit, int &difficulty, int &initialLives)
{
  if(my_window->IsOpened())
  {
    sf::String sSoundLevel = String();
    sSoundLevel.SetPosition(50, 100);

    sf::String sDifficulty = String();
    sDifficulty.SetPosition(50, 150);

    sf::String sLives = String();
    sLives.SetPosition(50, 200);

    sf::String sLanguage = String();
    sLanguage.SetPosition(50, 250);

    sf::String sSoundPack = String();
    sSoundPack.SetPosition(50, 300);

    sf::String sDifficultyLevel = String();
    sDifficultyLevel.SetPosition(500, 150);
    sDifficultyLevel.SetColor(sf::Color::Black);

    sf::String sMyLanguage = String();
    sMyLanguage.SetPosition(500, 250);
    sMyLanguage.SetColor(sf::Color::Black);

    sf::String sSelectedSoundPack = String();
    sSelectedSoundPack.SetPosition(500, 300);
    sSelectedSoundPack.SetColor(sf::Color::Black);

    sf::Shape sLifeCircle = sf::Shape::Circle(0,0, 10, sf::Color::White);
    sf::Shape sSoundRect;
    sf::Shape sSoundRectFond = sf::Shape::Rectangle(500, sSoundLevel.GetPosition().y +10, 700,sSoundLevel.GetPosition().y + 20, Color(188,188,188), 3, Color(125,125,125)); 

    Event event;
    bool backToMenu = false;

    int choice = 0;

    while(!backToMenu)
    {
      my_window->Clear(sf::Color::White);
      updateVolume();
      updateLanguage();
      updateSound();

      sSoundLevel.SetText(my_strings[SOUNDLEVEL] + " : ");
      sDifficulty.SetText(my_strings[DIFFICULTY] + " : ");
      sLives.SetText(my_strings[LIVESNUMBER] + " : ");
      sLanguage.SetText(my_strings[LANGUAGE] + " : ");
      sSoundPack.SetText(my_strings[SOUNDPACK] + " : " );

      sSoundLevel.SetColor(sf::Color::Black);
      sDifficulty.SetColor(sf::Color::Black);
      sLives.SetColor(sf::Color::Black);
      sLanguage.SetColor(sf::Color::Black);
      sSoundPack.SetColor(sf::Color::Black);

      switch(choice)
      {
        case 0 : sSoundLevel.SetColor(sf::Color::Red); break;
        case 1 : sDifficulty.SetColor(sf::Color::Red); break;
        case 2 : sLives.SetColor(sf::Color::Red);break;
        case 3 : sLanguage.SetColor(sf::Color::Red); break;
        case 4 : sSoundPack.SetColor(sf::Color::Red); break;
      }

      my_window->Draw(sSoundLevel);
      my_window->Draw(sDifficulty);
      my_window->Draw(sLives);
      my_window->Draw(sLanguage);
      my_window->Draw(sSoundPack);


      sSoundRect = sf::Shape::Rectangle(500, sSoundLevel.GetPosition().y + 10, 500 + 2 * my_soundLevel, sSoundLevel.GetPosition().y + 20, sf::Color::Black);
      my_window->Draw(sSoundRectFond);
      my_window->Draw(sSoundRect);

      sLifeCircle.SetColor(sf::Color::Black);
      for(int i=0; i < initialLives; i++)
      {
        sLifeCircle.SetPosition( 500 + 30*i, 215);
        my_window->Draw(sLifeCircle);
      }

      switch(difficulty)
      {
        case 1 : sDifficultyLevel.SetText(my_strings[NORMAL]); break;
        case 2 : sDifficultyLevel.SetText(my_strings[HARD]); break;
        case 3 : sDifficultyLevel.SetText(my_strings[VERYHARD]); break;
      }

      my_window->Draw(sDifficultyLevel);

      sMyLanguage.SetText(my_strings[MYLANGUE]);
      my_window->Draw(sMyLanguage);

      switch(my_soundpack)
      {
        case 1 : sSelectedSoundPack.SetText("8-Bit"); break;
        case 2 : sSelectedSoundPack.SetText("Human Mouth"); break;
      }

      my_window->Draw(sSelectedSoundPack);

      while(my_window->GetEvent(event))
      {
        if ((event.Type == Event::Closed) ||
          ((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Escape)))
        {
          my_window->Close();
          backToMenu = true;
          quit = true;
        }

        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Up))
          if(choice > 0)
            choice--;

        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Down))
          if(choice < 4)
            choice++;

        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Right))
        {
          switch(choice)
          {
            case 0 : if(my_soundLevel < 100) my_soundLevel += 20; break;
            case 1 : if(difficulty < 3) difficulty++; break;
            case 2 : if(initialLives < 5) initialLives++; break;
            case 3 : if(my_langue < 2) my_langue++; break;
            case 4 : if(my_soundpack < 2) my_soundpack++; break;
          }
        }

        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Left))
        {
          switch(choice)
          {
            case 0 : if(my_soundLevel > 0) my_soundLevel -= 20; break;
            case 1 : if(difficulty > 1) difficulty--; break;
            case 2 : if(initialLives > 1) initialLives--; break;
            case 3 : if(my_langue > 1) my_langue--; break;
            case 4 : if(my_soundpack > 1 ) my_soundpack-- ; break;
          }
        }


        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Return))
          backToMenu = true;
      }

      my_window->Display();
    }
  }
}

void GameView::viewHighscores(bool & quit)
{
  if(my_window->IsOpened())
  {
    my_window->Clear(sf::Color::White);

    sf::String sfString;
    sfString.SetColor(sf::Color::Black);

    ifstream f("highScores.txt");
    if(f)
    {
      string score, name;

      int y = 0;

      while(f >> name && f >> score) 
      {
        y += 30;

        sfString.SetText(name);
        sfString.SetPosition(50,y);
        my_window->Draw(sfString);

        sfString.SetText(score);
        sfString.SetPosition(400,y);
        my_window->Draw(sfString);
      }
    }
    else
    {
      sfString.SetText(my_strings[NOSCORE]);
      sfString.SetPosition(VIEW_WIDTH/2 - sfString.GetRect().GetWidth()/2,VIEW_HEIGHT/2 - sfString.GetRect().GetHeight()/2);
      my_window->Draw(sfString);
    }

    my_window->Display();

    Event event;
    bool backToMenu = false;
    while(my_window->GetEvent(event) || !backToMenu)
    {
        if(  /*(event.Type == Event::Closed) ||*/ /* < (BUG :) Cette partie de la condition fait quitter le jeu à l'affichage des scores, pour une raison inconnue. (On ne peut du coup pas fermer la fenêtre de façon "normale" si l'on affiche les "High Scores". */
          ((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Escape)))
        {
          my_window->Close();
          backToMenu = true;
          quit = true;
        }
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Return))
          backToMenu = true;
    }
  }
}

void GameView::askName()
{
  if(my_window->IsOpened())
  {
    sf::String s = String();

    string name;

    Event event;
    bool finished = false;

    while(!finished)
    {
      my_window->Clear();

      s.SetText(my_strings[ENTERNAME] + " : ");
      s.SetPosition(200, 200);
      s.SetColor(sf::Color::White);
      my_window->Draw(s);

      while(my_window->GetEvent(event))
      {
        if(event.Type == Event::TextEntered && name.size() < 16
          && event.Text.Unicode > 30 && (event.Text.Unicode < 127 || event.Text.Unicode > 159) && event.Text.Unicode != 32)
            name += event.Text.Unicode; 

        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Back) && name.size() > 0)
          name.erase(name.size()-1);
        if((event.Type == Event::KeyPressed) && (event.Key.Code == sf::Key::Return))
          finished = true;
      } 

      s.SetText(name);
      s.SetPosition(300,250);
      my_window->Draw(s);

      my_window->Display();
    }

    my_model->saveScore(name);
  }
}
