#ifndef GRAPHICELEMENT_H
#define GRAPHICELEMENT_H

/**
 * \file GraphicElement.h
 * \brief Déclaration de la classe GraphicElement
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */
 
#include <SFML/Graphics.hpp>

/**
 * \class GraphicElement GraphicElement.h
 * \brief Classe représentant les éléments à afficher
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class GraphicElement : public sf::Sprite 
{
	private:
		int _w, _h;
		bool _visible;
		
	public:
	
		/**
		 * \brief Constructeur par défaut
		 */
		GraphicElement();
		
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param image : Image à charger
		 * \param x : Abscisse de l'emplacement de l'élément à afficher dans la Vue
		 * \param y : Ordonnée de l'emplacement de l'élément à afficher dans la Vue
		 * \param w : Largeur de l'image à charger
		 * \param h : Hauteur de l'image à charger
		 */
		GraphicElement(sf::Image * image, int x, int y, int w, int h);

		/**
		 * \brief Destructeur
		 */  
		virtual ~GraphicElement();

		
		/**
		 * \brief Dessiner l'élément à afficher
		 *
		 * Méthode permettant de dessiner l'élément à afficher dans une fenêtre
		 *
		 * \param _window : Fenêtre où dessiner l'élément à afficher
		 */
		virtual void draw(sf::RenderWindow * _window);
		
  
		/**
		 * \brief Modifier la position de l'élément à afficher
		 * \param pos : Vecteur contenant les nouvelles coordonnées de l'élément à afficher
		 */
		void setPosition(const sf::Vector2f & pos);
		
		/**
		 * \brief Modifier la position de l'élément à afficher
		 * \param x : Nouvelle abscisse de l'élément à afficher
		 * \param y : Nouvelle ordonnée de l'élément à afficher
		 */
		void setPosition(int x, int y);
		
		/**
		 * \brief Modifier les dimensions de l'élément à afficher
		 * \param w : Nouvelle largeur de l'élément à afficher
		 * \param h : Nouvelle hauteur de l'élément à afficher
		 */
		void resize(int w, int h);

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de _visible
		 *
		 * \return : La visibilité actuelle de l'élément
		 */
		bool getVisible() const;
		
		/**
		 * \brief Définir la visibilité de l'élément à afficher
		 * \param visible : Nouvelle visibilité de l'élément à afficher
		 */
		void setVisible(bool visible);
};
#endif

