#ifndef HORLOGE_H
#define HORLOGE_H

/**
 * \file Horloge.h
 * \brief Déclaration de la classe Horloge
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

#include <ctime>

/**
 * \class Horloge Horloge.h
 * \brief Classe représentant le temps dans le jeu
 *
 * Remplace la classe Clock de la SFML 1.6, afin notamment d'implémenter une pause
 *
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class Horloge
{
	private:
		time_t m_startTime;
		timespec tp;
		float m_buffer;
		bool m_paused;
	public:
	
		/**
		 * \brief Constructeur par défaut
		 */
		Horloge();
		
		/**
		 * \brief Retourner le temps écoulé
		 * \return : Le temps écoulé depuis le démarrage de l'horloge
		 */
		float getTime();
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de m_paused
		 *
		 * \return : L'état de l'horloge (\b true si l'horloge est en pause, \b false sinon)
		 */
		bool getStatus() const;

		
		/**
		 * \brief Démarrer l'horloge
		 */
		void start();
		
		/**
		 * \brief Mettre l'horloge en pause
		 */
		void pause();
		
		/**
		 * \brief Arrêter l'horloge
		 *
		 * Met l'horloge en pause et la réinitialise
		 *
		 */
		void stop();
		
		/**
		 * \brief Réinitialiser l'horloge
		 */
		void reset();


};


#endif

