#ifndef INTERFACE_H
#define INTERFACE_H

/**
 * \file Interface.h
 * \brief Déclaration de la classe Interface
 * \author Quentin HARSCOËT
 */

#include <SFML/Graphics.hpp>
#include <string>

#include "GameModel.h"
#include "GraphicElement.h"
#include "constantes.h"

/**
 * \class Interface Interface.h
 * \brief Classe représentant le HUD (affichage tête haute)
 * \author Quentin HARSCOËT
 */

class Interface
{
	private:
		//HUD
		sf::String my_HUDsfStrings[6];

		sf::Image* my_revive_image;
		GraphicElement my_revive_sprite_template;
		int my_nbVies;

		sf::Shape my_rectPV;
		sf::Shape my_rectPVFond;

		sf::Shape my_rectBossPV;

		sf::Font my_font;
	public:
	
		/**
		 * \brief Constructeur par défaut
		 */
		Interface();

		/**
		 * \brief Destructeur
		 */  
		~Interface();

		
		/**
		 * \brief Dessiner le HUD
		 *
		 * Méthode permettant de dessiner le HUD dans une fenêtre
		 *
		 * \param _window : Fenêtre où dessiner le HUD
		 */
		void drawHUD(sf::RenderWindow * _window);

		
		/**
		 * \brief Mettre à jour le HUD
		 *
		 * \param model : Modèle du jeu
		 * \param strings[] : Tableau contenant les textes du HUD (utilisé pour la traduction)
		 */
		void update(GameModel* model, string strings[]);
		
		/**
		 * \brief Mettre à jour la barre de vie du boss
		 *
		 * \param boss : Boss affronté
		 */
		void updateBossPV(Boss* boss);


};

#endif
