#include "Joueur.h"
#include "GameModel.h"
#include "constantes.h"

#include <sstream>

using namespace std;


Joueur::Joueur(int power, TypeTir* typeTir,int vies, int pvMax, int bombs, float shootingSpeed)
:Vaisseau(pvMax, typeTir, shootingSpeed), my_vies(vies), my_power(power),my_bombs(bombs)
{
	my_dx = my_dy = SHEEP_SPEED;
	my_h = SHEEP_HEIGHT * SHEEP_SCALE;
	my_w = SHEEP_WIDTH * SHEEP_SCALE;
	my_x = MODEL_WIDTH/2 - my_w/2;
	my_y = MODEL_HEIGHT - my_h;

	my_numberOfShoots = 1;
}

void Joueur::moveRight()
{if(my_x + my_dx + my_w < MODEL_WIDTH)
	my_x += my_dx;	
else
	my_x = MODEL_WIDTH - my_w;}
void Joueur::moveLeft()
{if(my_x - my_dx > 0)
	my_x -= my_dx;
else
	my_x = 0;}
void Joueur::moveDown()
{if(my_y + my_dy + my_h < MODEL_HEIGHT)
	my_y += my_dy;
else
	my_y = MODEL_HEIGHT - my_h;}
void Joueur::moveUp()
{if(my_y - my_dy > 0)
	my_y -= my_dy;
else
	my_y = 0;}



void Joueur::shoot(GameModel* model)
{
	if(my_clock.GetElapsedTime() > my_shootingSpeed)
	{
		switch(my_numberOfShoots)
		{
			case 1:
			{	Tir* t = new Tir(my_x, my_y, 0, -my_typeTir->getSpeed(),1, my_typeTir);
			    t->setX(my_x + my_w/2 - t->getW()/2);
				model->addTir(t);	}
				break;
			case 2:
			{	
				Tir* t1 = new Tir(my_x, my_y, 0, -my_typeTir->getSpeed(),1, my_typeTir);
				Tir* t2 = new Tir(my_x, my_y, 0, -my_typeTir->getSpeed(),1, my_typeTir);
				t2->setX(my_x + my_w );
				model->addTir(t1);
				model->addTir(t2);	
			}
				break;
			case 3:
			{	Tir* T1 = new Tir(my_x, my_y, 0, -my_typeTir->getSpeed(),1, my_typeTir);
				Tir* T2 = new Tir(my_x, my_y, 0, -my_typeTir->getSpeed(),1, my_typeTir);
				Tir* T3 = new Tir(my_x, my_y, 0, -my_typeTir->getSpeed(),1, my_typeTir);
				T2->setX(my_x + my_w/2 - T2->getW()/2);
				T3->setX(my_x + my_w - T3->getW());
				model->addTir(T1);
				model->addTir(T2);
				model->addTir(T3);	}
				break;
		}

		my_clock.Reset();
	}
}

vector<Tir*>::iterator Joueur::isShot(GameModel* model) const
{
	vector<Tir*>::iterator shot = model->getVectorTir()->end();
    vector<Tir*>::iterator it = model->getVectorTir()->begin();
    while( it != model->getVectorTir()->end() && shot == model->getVectorTir()->end())
    {
        if((*it)->getX() + (*it)->getW() > my_x && (*it)->getX() < my_x + my_w && (*it)->getY() + (*it)->getH() > my_y && (*it)->getY() < my_y+my_h)
        {
            if(!(*it)->getPlayer())
                shot = it; 
        }  
        it++;
    }

   return shot;
}

bool Joueur::isHitByEnemy(GameModel* model)
{
	bool found = false;
	vector<Ennemi*>::iterator it = model->getVectorEnnemis()->begin();
	while(it != model->getVectorEnnemis()->end() && !found)
	{
		if((*it)->getX() + (*it)->getW() > my_x && (*it)->getX() < my_x + my_w && (*it)->getY() + (*it)-> getH() >  my_y && (*it)->getY() < my_y + my_h)
			if((*it)->getAlive())
				found = true;

		it++;
	}

	return found;
}

void Joueur::useBomb(GameModel* model)
{
	if(my_bombs > 0)
	{
		for(vector<Ennemi*>::iterator it = model->getVectorEnnemis()->begin(); it!=model->getVectorEnnemis()->end(); it++)
		{	Boss* b = dynamic_cast<Boss*>(*it);
			if(b == NULL)
			{
				model->setScore(model->getScore() + (*it)->getScore());
				(*it)->setAlive(false);
			}
		}
		for(vector<Tir*>::iterator it = model->getVectorTir()->begin(); it != model->getVectorTir()->end(); it++)
		{	model->getDeletedItems()->push_back(*it);
			delete *it;	}
		model->getVectorTir()->clear();

		my_bombs--;
	}
}

void Joueur::addPower(int power, GameModel* model)
{
	my_power += power;
	if(my_power >= 500 && my_numberOfShoots < 3)
		my_numberOfShoots = 3;
	else if(my_power >= 300 )
	{	
		if( my_typeTir != model->getTirTypes()[2])
			my_typeTir = model->getTirTypes()[2];
	}
	else if(my_power >= 200 && my_numberOfShoots < 2)
		my_numberOfShoots = 2;
	else if(my_power >= 100 )
	{	
		if(my_typeTir != model->getTirTypes()[1])
			my_typeTir = model->getTirTypes()[1];
	}
}

void Joueur::addHP(int hp)
{
	if(getHP() + hp <= getMaxHP())
		setHP(getHP() + hp); 
	else
		setHP(getMaxHP());
}



//Accesseurs
int Joueur::getVies() const
{return my_vies;	}
int Joueur::getPower() const
{   return my_power;    }
int Joueur::getBombs() const
{return my_bombs;	}
int Joueur::getNumberOfShoots() const
{return my_numberOfShoots;	}
void Joueur::setVies(int vies)
{my_vies = vies;	}
void Joueur::setBombs(int bombs)
{my_bombs = bombs;	}
void Joueur::setPower(int i)
{    my_power = i;  }
void Joueur::setNumberOfShoots(int n)
{ my_numberOfShoots = n;	}


string Joueur::getStringBombs() const
{
	stringstream str;
	str << my_bombs << endl;
	return str.str();
}
string Joueur::getStringPower() const
{
	stringstream str;
	str << my_power << endl;
	return str.str();
}




