#ifndef MOVABLE_ELEMENT_H
#define MOVABLE_ELEMENT_H

/**
 * \file MovableElement.h
 * \brief Déclaration de la classe MovableElement
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

/**
 * \class MovableElement MovableElement.h
 * \brief Classe représentant les éléments mobiles
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class MovableElement
{
	protected :
		int my_x; /**< Abscisse de l'élément mobile */
		int my_y; /**< Ordonnée de l'élément mobile */
		int my_dx; /**< Vitesse (abscisse) de l'élément mobile */
		int my_dy; /**< Vitesse (ordonnée) de l'élément mobile */
		int my_w; /**< Largeur de l'élément mobile */
		int my_h; /**<  Hauteur de l'élément mobile*/
		
		
	public :
	
		/**
		 * \brief Constructeur par défaut
		 */
		MovableElement();
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param x : Abscisse de l'emplacement de l'élément mobile dans le Modéle
		 * \param y : Ordonnée de l'emplacement de l'élément mobile dans le Modéle
		 * \param dx : Vitesse (abscisse) de l'élément mobile
		 * \param dy : Vitesse (ordonnée) de l'élément mobile
		 * \param w : Largeur de l'élément mobile
		 * \param h : Hauteur de l'élément mobile
		 */
		MovableElement(int x, int y, int dx, int dy, int w, int h);

		/**
		 * \brief Destructeur
		 */  
		virtual ~MovableElement();


		/**
		 * \brief Déplacer l'élément mobile vers la gauche
		 */ 
		virtual void moveLeft();

		/**
		 * \brief Déplacer l'élément mobile vers la droite
		 */ 
		virtual void moveRight();

		/**
		 * \brief Déplacer l'élément mobile vers le bas
		 */ 
		virtual void moveDown();

		/**
		 * \brief Déplacer l'élément mobile vers le haut
		 */ 
		virtual void moveUp();


		/**
		 * \brief Déplacer l'élément mobile
		 *
		 * Méthode permettant de déplacer l'élément mobile en abscisse et en ordonnée
		 */ 
		virtual void move();
		
		//Accesseurs
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_x
		 *
		 * \return : L'abscisse de l'élément mobile
		 */
		int getX() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_y
		 *
		 * \return : L'ordonnée de l'élément mobile
		 */
		int getY() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_dy
		 *
		 * \return : La vitesse (ordonnée) de l'élément mobile
		 */
		int getDY() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_w
		 *
		 * \return : La largeur de l'élément mobile de l'élément mobile
		 */
		int getW() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_h
		 *
		 * \return : La hauteur de l'élément mobile de l'élément mobile
		 */
		int getH() const ;
		
		/**
		 * \brief Définir l'abscisse de l'élément mobile
		 */
		void setX(int x);
		
		/**
		 * \brief Définir l'ordonnée de l'élément mobile
		 */
		void setY(int y);

};

#endif
