#ifndef NIVEAU_H
#define NIVEAU_H

/**
 * \file Niveau.h
 * \brief Déclaration de la classe Niveau
 * \author Quentin HARSCOËT
 */
 
#include <map>
#include "TypeEnnemi.h"

class GameModel;

/**
 * \class Niveau Niveau.h
 * \brief Classe représentant les niveaux du jeu
 * \author Quentin HARSCOËT
 */

class Niveau
{
	private :

		int idNiveau;
		std::multimap<float, TypeEnnemi*> ennemis; // float : Instant (en secondes) où l'ennemi apparait, TypeEnnemi* : Le type de l'ennemi

	public : 
	
		/**
		 * \brief Constructeur par défaut
		 */
		Niveau();
	
		/**
		 * \brief Initialise le niveau
		 * \param model : Modèle du jeu
		 */
		void initNiveau(GameModel* model);
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de ennemis
		 *
		 * \return : L'adresse de la multimap contenant les instant où les ennemis appraîssent, ainsi que les types de ces derniers
		 */
		const std::multimap<float, TypeEnnemi*>* getMap() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de idNiveau
		 *
		 * \return : L'identifiant du niveau
		 */
		int getID() const;
};




#endif
