#ifndef TIR_H
#define TIR_H

/**
 * \file Tir.h
 * \brief Déclaration de la classe Tir
 * \author Quentin HARSCOËT
 */

#include "MovableElement.h"
#include "TypeTir.h"

/**
 * \class Tir Tir.h
 * \brief Classe représentant les tirs des Vaisseau
 * \author Quentin HARSCOËT
 */

class Tir : public MovableElement
{
	private :
		bool my_fromPlayer;
		TypeTir* my_type;

	public :
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param x : Abscisse de l'emplacement du tir dans le Modéle
		 * \param y : Ordonnée de l'emplacement du tir dans le Modéle
		 * \param fromPlayer : Origine du tir (\b true si le tir vient du joueur, \b false sinon)
		 * \param type : Type du tir
		 */
		Tir(int x, int y, bool fromPlayer, TypeTir* type);
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param x : Abscisse de l'emplacement du tir dans le Modéle
		 * \param y : Ordonnée de l'emplacement du tir dans le Modéle
		 * \param dx : Vitesse (abscisse) du tir
		 * \param dy : Vitesse (ordonnée) du tir
		 * \param fromPlayer : Origine du tir (\b true si le tir vient du joueur, \b false sinon)
		 * \param type : Type du tir
		 */
		Tir(int x, int y, int dx, int dy, bool fromPlayer, TypeTir* type);

		/**
		 * \brief Destructeur
		 */  
		virtual ~Tir();
		
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_fromPlayer
		 *
		 * \return : L'origine du tir (\b true si le tir vient du joueur, \b false sinon)
		 */
		bool getPlayer() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_type
		 *
		 * \return : Le type du tir
		 */
		TypeTir* getType() const;
};

#endif
