#ifndef TYPE_BONUS_H
#define TYPE_BONUS_H

/**
 * \file TypeBonus.h
 * \brief Déclaration de la classe TypeBonus et de l'énumération BonusEffect
 * \author Quentin HARSCOËT
 */

#include "constantes.h"

/**
 * \struct BonusEffect
 * \brief Catégories de bonus
 */
 
enum BonusEffect 
{ 
	POWER, /**< Gain de puissance */
	LIFE, /**< Vie supplémentaire */
	BOMB, /**< Bombe supplémentaire */
	HP /**< Gain de points de vie */
};

/**
 * \class TypeBonus TypeBonus.h
 * \brief Classe représentant les différents types de Bonus
 * \author Quentin HARSCOËT
 */

class TypeBonus
{
	private:	
		BonusEffect my_effect;
		int my_value;
		float my_frequency; // Fréquence/probabilité d'apparition, comprise entre 0 et 1
		float my_scale; // Échelle du Bonus (afin d'en avoir des gros et petits avec le même BonusEffect)

	public:
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param effect : Effet/Catégorie du Bonus
		 * \param value : Valeur du Bonus (le gain apporté)
		 * \param frequency : Fréquence/probabilité d'apparition du bonus, comprise entre 0 et 1
		 * \param scale : Taille du Bonus
		 */
		TypeBonus(BonusEffect effect, int value, float frequency, float scale);

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_effect
		 *
		 * \return : L'effet/catégorie du Bonus
		 */
		BonusEffect getEffect() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_value
		 *
		 * \return : La valeur du Bonus (le gain apporté)
		 */
		int getValue() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_frequency
		 *
		 * \return : La fréquence/probabilité d'apparition du Bonus
		 */
		float getFrequency() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_scale
		 *
		 * \return : La taille du Bonus
		 */
		float getScale() const;
};	

#endif
