#ifndef TYPE_BOSS_H
#define TYPE_BOSS_H

/**
 * \file TypeBoss.h
 * \brief Déclaration de la classe TypeBoss
 * \author Quentin HARSCOËT
 */

#include "TypeEnnemi.h"

/**
 * \class TypeBoss TypeBoss.h
 * \brief Classe représentant les différents types de Boss
 * \author Quentin HARSCOËT
 */

class TypeBoss : public TypeEnnemi
{
	private:
		float my_changePatternDelay;

	public:
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param score : Points rapportés par le Boss
		 * \param speed : Vitesse du Boss
		 * \param shootingSpeed : Vitesse de tir du Boss
		 * \param hp : Points de vie du Boss
		 * \param w : Largeur du Boss
		 * \param h : Hauteur du Boss
		 * \param typeTir : Type de tir du Boss
		 * \param shootPattern : Pattern de tir du Boss
		 * \param filename : Nom du fichier du sprite du Boss
		 * \param changePatternDelay : Intervalle de temps entre deux changements de pattern du Boss
		 */
		TypeBoss(int score, int speed, float shootingSpeed, int hp, int w, int h, TypeTir* typeTir, ShootPattern shootPattern, std::string filename,
				 float changePatternDelay);

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_changePatternDelay
		 *
		 * \return : L'intervalle de temps entre deux changements de pattern du Boss
		 */
		float getChangePatternDelay() const;

};

#endif
