#ifndef TYPE_ENNEMI_H
#define TYPE_ENNEMI_H

/**
 * \file TypeEnnemi.h
 * \brief Déclaration de la classe TypeEnnemi et de l'énumération ShootPattern
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

#include <string>
#include "TypeTir.h"

/**
 * \struct ShootPattern
 * \brief Catégories de patterns
 */

enum ShootPattern 
{
	SIMPLE, /**< Tir simple en ligne droite */
	M_TRIPLE, /**< Tir triple, "en triangle" */
	TOPLAYER,  /**< Tir dirigé vers le Joueur */
	HALFCIRCLE /**< Tir en demi-cercle, devant le Vaisseau */
};

/*	M-TRIPLE =  S
			   /|\	
			  / | \
*/

/**
 * \class TypeEnnemi TypeEnnemi.h
 * \brief Classe représentant les différents types d'Ennemi
 * \author Théo CHASSAIGNE
 * \author Quentin HARSCOËT
 */

class TypeEnnemi
{
	private : 
		std::string my_sprite_filename;
		int my_score;
		int my_speed;
		float my_shootingSpeed; // cadence de tir, en secondes (tir 1 fois toutes les x secondes)
		int my_hp;
		int my_h;
		int my_w;
		TypeTir* my_typeTir;
		ShootPattern my_shootPattern;

	public :
	
		/**
		 * \brief Constructeur par défaut
		 */
		TypeEnnemi();
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param score : Points rapportés par l'Ennemi
		 * \param speed : Vitesse de l'Ennemi
		 * \param shootingSpeed : Vitesse de tir de l'Ennemi
		 * \param hp : Points de vie de l'Ennemi
		 * \param w : Largeur de l'Ennemi
		 * \param h : Hauteur de l'Ennemi
		 * \param typeTir : Type de tir de l'Ennemi
		 * \param shootPattern : Pattern de tir de l'Ennemi
		 * \param filename : Nom du fichier du sprite de l'Ennemi
		 */
		TypeEnnemi(int score, int speed, float shootingSpeed, int hp, int w, int h, TypeTir* typeTir, ShootPattern shootPattern, std::string filename);

		/**
		 * \brief Destructeur
		 */
		virtual ~TypeEnnemi();
		
		//Accesseurs

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_score
		 *
		 * \return : Les points rapportés par l'Ennemi
		 */
		int getScore() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_speed
		 *
		 * \return : La vitesse de l'Ennemi
		 */
		int getSpeed() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_shootingSpeed
		 *
		 * \return : La vitesse de tir de l'Ennemi
		 */
		float getShootingSpeed() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_hp
		 *
		 * \return : Les points de vie de l'Ennemi
		 */
		int getHP() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_w
		 *
		 * \return : La largeur de l'Ennemi
		 */
		int getW() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_h
		 *
		 * \return : La hauteur de l'Ennemi
		 */
		int getH() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_filename
		 *
		 * \return : La nom du fichier du sprite de l'Ennemi
		 */
		std::string getFilename() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_typeTir
		 *
		 * \return : Le type de tir de l'Ennemi
		 */
		TypeTir* getTypeTir() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_shootPattern
		 *
		 * \return : Le pattern de tir de l'Ennemi
		 */
		ShootPattern getShootPattern() const;


};

#endif
