#ifndef TYPE_TIR_H
#define TYPE_TIR_H

/**
 * \file TypeTir.h
 * \brief Déclaration de la classe TypeTir
 * \author Quentin HARSCOËT
 */

#include <string>

/**
 * \class TypeTir TypeTir.h
 * \brief Classe représentant les différents types de Tir
 * \author Quentin HARSCOËT
 */

class TypeTir
{
	private:

		int my_w;
		int my_h;
		int my_damages;
		int my_speed;
		std::string my_sprite_filename;

	public :
	
		/**
		 * \brief Constructeur paramétré
		 *
		 * \param w : Largeur du Tir
		 * \param h : Hauteur du Tir
		 * \param damages : Dommages infligés par le Tir
		 * \param speed : Vitesse du Tir
		 * \param filename : Nom du fichier du sprite du Tir
		 */
		TypeTir(int w, int h, int damages, int speed, std::string filename);

		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_filename
		 *
		 * \return : Le nom du fichier du sprite du Tir
		 */
		std::string getFilename() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_h
		 *
		 * \return : La hauteur du Tir
		 */
		int getH() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_w
		 *
		 * \return : La largeur du Tir
		 */
		int getW() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_w
		 *
		 * \return : Les dommages infligés par le Tir
		 */
		int getDamages() const;
		
		/**
		 * \brief Accesseur
		 *
		 * Accesseur de my_w
		 *
		 * \return : La vitesse du Tir
		 */
		int getSpeed() const;
};


#endif
